import sys
from os import getenv
from pathlib import Path


# create unsupported.txt in os specific user directory
# doc: https://github.com/ActiveState/appdirs/blob/master/appdirs.py#L44 derivative
def user_data_dir(file_name):
    r"""
    Get OS specific data directory path for SwagLyrics.

    Typical user data directories are:
        macOS:    ~/Library/Application Support/SwagLyrics
        Unix:     ~/.local/share/SwagLyrics   # or in $XDG_DATA_HOME, if defined
        Win 10:   C:\Users\<username>\AppData\Local\SwagLyrics
    For Unix, we follow the XDG spec and support $XDG_DATA_HOME if defined.
    :param file_name: file to be fetched from the data dir
    :return: full path to the user-specific data dir
    """
    # get os specific path
    if sys.platform.startswith("win"):
        os_path = getenv("LOCALAPPDATA")
    elif sys.platform.startswith("darwin"):
        os_path = "~/Library/Application Support"
    else:
        # linux
        os_path = getenv("XDG_DATA_HOME", "~/.local/share")

    # join with SwagLyrics dir
    path = Path(os_path) / "SwagLyrics"

    return path.expanduser() / file_name


name = 'swaglyrics'
__version__ = '1.2.2'
backend_url = 'https://api.swaglyrics.dev'
api_timeout = 10
genius_timeout = 20
unsupported_txt = user_data_dir("unsupported.txt")

# create unsupported.txt if it doesn't exist
unsupported_txt.parent.mkdir(parents=True, exist_ok=True)
unsupported_txt.touch(exist_ok=True)


class SameSongPlaying(Exception):
    pass



import os

from SwSpotify import spotify, SpotifyNotRunning
from flask import Flask, render_template

from swaglyrics import SameSongPlaying
from swaglyrics.cli import lyrics

app = Flask(__name__, template_folder=os.path.join(os.path.dirname(os.path.abspath(__file__)), 'templates'))
# use relative path of the template folder

song = None
artist = None


@app.route('/')
def tab() -> str:
    # format lyrics for the browser tab template
    global song, artist
    try:
        song, artist = spotify.current()
        current_lyrics = lyrics(song, artist)
    except SpotifyNotRunning:
        current_lyrics = 'Nothing playing at the moment.'
    lyrics_lines = current_lyrics.split('\n')  # break lyrics line by line
    return render_template('lyrics.html', lyrics=lyrics_lines, song=song, artist=artist)


@app.route('/songChanged', methods=['GET'])
def song_changed() -> str:
    # to refresh lyrics when song changed
    global song, artist
    try:
        if spotify.current() == (song, artist):
            raise SameSongPlaying
        else:
            return 'yes'
    except (SpotifyNotRunning, SameSongPlaying):
        return 'no'


if __name__ == '__main__':
    app.run()



import os
import re
from typing import Optional

import requests
from bs4 import BeautifulSoup, UnicodeDammit
from colorama import init, Fore, Style
from html import unescape
from unidecode import unidecode

from swaglyrics import __version__, unsupported_txt, backend_url, api_timeout, genius_timeout


def clear() -> None:
    os.system('cls' if os.name == 'nt' else 'clear')  # clear command window


# matches braces with feat included or text after -, also adds support for Bollywood songs by matching (From "<words>")
brc = re.compile(r'([(\[](feat|ft|From "[^"]*")[^)\]]*[)\]]|- .*)', re.I)
aln = re.compile(r'[^ \-a-zA-Z0-9]+')  # matches non space or - or alphanumeric characters
spc = re.compile(' *- *| +')  # matches one or more spaces
wth = re.compile(r'(?: *\(with )([^)]+)\)')  # capture text after with
nlt = re.compile(r'[^\x00-\x7F\x80-\xFF\u0100-\u017F\u0180-\u024F\u1E00-\u1EFF]')  # match only latin characters,
# built using latin character tables (basic, supplement, extended a,b and extended additional)


def stripper(song: str, artist: str) -> str:
    """
    Generate the url path given the song and artist to format the Genius URL with.
    Strips the song and artist of special characters and unresolved text such as 'feat.' or text within braces.
    Then concatenates both with hyphens replacing the blank spaces.
    Eg.
    >>>stripper('Paradise City', 'Guns n’ Roses')
    >>>'Guns-n-Roses-Paradise-City'
    Which then formats the url to https://genius.com/Guns-n-Roses-Paradise-City-lyrics
    :param song: currently playing song
    :param artist: song artist
    :return: formatted url path
    """
    song = re.sub(brc, '', song).strip()  # remove braces and included text with feat and text after '- '
    ft = wth.search(song)  # find supporting artists if any
    if ft:
        song = song.replace(ft.group(), '')  # remove (with supporting artists) from song
        ar = ft.group(1)  # the supporting artist(s)
        if '&' in ar:  # check if more than one supporting artist and add them to artist
            artist += f'-{ar}'
        else:
            artist += f'-and-{ar}'
    song_data = artist + '-' + song
    # swap some special characters
    url_data = song_data.replace('&', 'and')
    # replace /, !, _ with space to support more songs
    url_data = url_data.replace('/', ' ').replace('!', ' ').replace('_', ' ')
    for ch in ['Ø', 'ø']:
        if ch in url_data:
            url_data = url_data.replace(ch, '')
    url_data = re.sub(nlt, '', url_data)  # remove non-latin characters before unidecode
    url_data = unidecode(url_data)  # convert accents and other diacritics
    url_data = re.sub(aln, '', url_data)  # remove punctuation and other characters
    url_data = re.sub(spc, '-', url_data.strip())  # substitute one or more spaces to -
    return url_data


def get_lyrics(song: str, artist: str) -> Optional[str]:
    """
    Get lyrics from Genius given the song and artist.
    Formats the URL with the stripped url path to fetch the lyrics.
    :param song: currently playing song
    :param artist: song artist
    :return: song lyrics or None if lyrics unavailable
    """
    url_data = stripper(song, artist)  # generate url path using stripper()
    if url_data.startswith('-') or url_data.endswith('-'):
        return None  # url path had either song in non-latin, artist in non-latin, or both
    url = f'https://genius.com/{url_data}-lyrics'  # format the url with the url path
    try:
        page = requests.get(url, timeout=genius_timeout)
        page.raise_for_status()
    except requests.exceptions.HTTPError:
        url_data = requests.get(f'{backend_url}/stripper', data={
            'song': song,
            'artist': artist}, timeout=api_timeout).text
        if not url_data:
            return None
        url = 'https://genius.com/{}-lyrics'.format(url_data)
        page = requests.get(url, timeout=genius_timeout)

    html = BeautifulSoup(page.text, "html.parser")
    lyrics_path = html.find("div", class_="lyrics")  # finding div on Genius containing the lyrics
    if lyrics_path:
        lyrics = UnicodeDammit(lyrics_path.get_text().strip()).unicode_markup
    else:
        # hotfix!
        lyrics_path = html.find_all("div", class_=re.compile("^Lyrics__Container"))
        lyrics_data = []
        for x in lyrics_path:
            lyrics_data.append(UnicodeDammit(re.sub("<.*?>", "", str(x).replace("<br/>", "\n"))).unicode_markup)

        lyrics = "\n".join(unescape(lyrics_data))  # also convert escaped characters to symbols
    return lyrics


def lyrics(song: str, artist: str, make_issue: bool = True) -> str:
    """
    Displays the fetched lyrics if song playing and handles if lyrics unavailable.
    :param song: currently playing song
    :param artist: song artist
    :param make_issue: whether to make an issue on GitHub if song unsupported
    :return: lyrics if song playing
    """
    try:
        with open(unsupported_txt, encoding='utf-8') as unsupported:
            if f'{song} by {artist}' in unsupported.read():
                return f'Lyrics unavailable for {song} by {artist}.\n'
    except FileNotFoundError:
        pass
    init(autoreset=True)
    print(Fore.CYAN + Style.BRIGHT + f'\nGetting lyrics for {song} by {artist}.\n')
    lyrics = get_lyrics(song, artist)
    if not lyrics:
        lyrics = f"Couldn't get lyrics for {song} by {artist}.\n"
        # log song and artist for which lyrics couldn't be obtained
        with open(unsupported_txt, 'a', encoding='utf-8') as f:
            f.write(f'{song} by {artist} \n')
        if make_issue and re.search(aln, song + artist):
            # only runs if non space or non alphanumeric characters are present
            r = requests.post(f'{backend_url}/unsupported', data={
                'song': song,
                'artist': artist,
                'version': __version__
            }, timeout=api_timeout)
            if r.status_code == 200:
                lyrics += r.text
    return lyrics


if __name__ == '__main__':
    pass



import setuptools

import swaglyrics

with open("README.md", "r", encoding='utf8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="swaglyrics",
    version=swaglyrics.__version__,
    author="Aadi Bajpai",
    author_email="aadi@swaglyrics.dev",
    description="Fetch the currently playing song from Spotify and display the lyrics in terminal or a browser tab. "
                "Very fast.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SwagLyrics/SwagLyrics-For-Spotify",
    entry_points={'console_scripts': ['swaglyrics=swaglyrics.__main__:main']},
    packages=['swaglyrics'],
    license='MIT',
    include_package_data=True,
    install_requires=[
        'SwSpotify>=1.2.0',
        'flask==1.1.2',
        'requests>=2.24.0',
        'unidecode==1.1.1',
        'beautifulsoup4==4.9.1',
        'colorama==0.4.3'
    ],
    extras_require={
        'dev': [
            'mock',
            'pytest',
            'pytest-cov',
            'blinker',
            'Werkzeug==1.0.1',  # remove this once stuff's resolved
            'flask_testing',
            'mypy'
        ]
    },
    python_requires='>=3.6',
    keywords='spotify lyrics python genius',
    project_urls={
        'Documentation': 'https://swaglyrics.dev',
        'Funding': 'https://www.paypal.me/sendclash',
        'Source': 'https://github.com/SwagLyrics/SwagLyrics-For-Spotify',
        'Tracker': 'https://github.com/SwagLyrics/SwagLyrics-For-Spotify/issues',
    },
    classifiers=(
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Framework :: Flask",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: End Users/Desktop",
    ),
)



import argparse
import os
import sys
import time

import requests
from SwSpotify import spotify, SpotifyNotRunning

from swaglyrics import unsupported_txt, SameSongPlaying, __version__ as version, backend_url, api_timeout
from swaglyrics.cli import lyrics, clear
from swaglyrics.tab import app


def unsupported_precheck(force: bool = False) -> None:
    """
    Checks if new version available and updates unsupported.txt from server.
    Runs only if 24h have elapsed since previous run or if forced via the -u flag.
    """
    if not force:
        # check 24h has elapsed since last update check
        with open(unsupported_txt, 'r', encoding='utf-8') as f:
            try:
                last_updated = float(f.readline())
                if time.time() - last_updated < 86400:  # 86400 seconds in a day
                    return None
            except ValueError:  # would occur if first line string, on initial versions of unsupported.txt
                pass
            except PermissionError as e:
                print("You should install SwagLyrics as --user or use sudo to access unsupported.txt.", e)
                sys.exit(1)
    try:
        v = requests.get(f'{backend_url}/version')
        ver = v.text
        if ver > version:
            print("New version of SwagLyrics available: v{ver}\nPlease update :)".format(ver=ver))
            print("To update, execute pip install -U swaglyrics")
    except requests.exceptions.RequestException:
        pass
    print('Updating unsupported.txt from server.')
    with open(unsupported_txt, 'w', encoding='utf-8') as f:
        try:
            unsupported_songs = requests.get(f'{backend_url}/master_unsupported', timeout=api_timeout)
            last_updated = time.time()
            f.write(f'{last_updated}\n')
            f.write(unsupported_songs.text)
            print("Updated unsupported.txt successfully.")
        except requests.exceptions.RequestException as e:
            print("Could not update unsupported.txt successfully.", e)
        except PermissionError as e:
            print("You should install SwagLyrics as --user or use sudo to access unsupported.txt.", e)
            sys.exit(1)


def show_tab() -> None:
    from threading import Timer
    from webbrowser import open
    print('Firing up a browser tab!')
    app.template_folder = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'templates')
    app.static_folder = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'static')
    port = 5042  # random
    url = f"http://127.0.0.1:{port}"
    Timer(1.25, open, args=[url]).start()
    app.run(port=port)


def show_cli(make_issue: bool = False) -> None:
    try:
        song, artist = spotify.current()  # get currently playing song, artist
        print(lyrics(song, artist, make_issue))
        print('\n(Press Ctrl+C to quit)')
    except SpotifyNotRunning as e:
        print(e)
        print('\n(Press Ctrl+C to quit)')
        song, artist = None, None
    while True:
        # refresh every 5s to check whether song changed
        # if changed, display the new lyrics
        try:
            try:
                if spotify.current() == (song, artist):
                    raise SameSongPlaying
                else:
                    song, artist = spotify.current()
                    clear()
                    print(lyrics(song, artist, make_issue))
                    print('\n(Press Ctrl+C to quit)')
            except (SpotifyNotRunning, SameSongPlaying):
                time.sleep(5)
        except KeyboardInterrupt:
            print('\nSure boss, exiting.')
            sys.exit()


def main() -> None:
    # print(r"""
    #  ____                     _               _
    # / ___|_      ____ _  __ _| |   _   _ _ __(_) ___ ___
    # \___ \ \ /\ / / _` |/ _` | |  | | | | '__| |/ __/ __|
    #  ___) \ V  V / (_| | (_| | |__| |_| | |  | | (__\__ \
    # |____/ \_/\_/ \__,_|\__, |_____\__, |_|  |_|\___|___/
    #                     |___/      |___/
    # 	""")
    # print('\n')

    parser = argparse.ArgumentParser(
        description="Get lyrics for the currently playing song on Spotify. Either --tab or --cli is required.")

    parser.add_argument('-t', '--tab', action='store_true', help='Display lyrics in a browser tab.')
    parser.add_argument('-c', '--cli', action='store_true', help='Display lyrics in the command-line.')
    parser.add_argument('-n', '--no-issue', action='store_false', help='Disable issue-making on cli.')
    parser.add_argument('-u', '--update-check', action='store_true', help='Force check for updates.')
    args = parser.parse_args()

    if args.tab:
        unsupported_precheck(args.update_check)
        show_tab()

    elif args.cli:
        unsupported_precheck(args.update_check)
        make_issue = args.no_issue
        show_cli(make_issue)
    else:
        parser.print_help()


if __name__ == '__main__':
    main()
